import { parse } from 'accept-language-parser';
import { loadTranslations, setLocale, setRoute } from '$lib/translations';
import { SUPPORTED_LOCALES } from '$lib/translations';

/**
 * A set of all supported locale codes, used to validate and match against
 * user preferences from cookies or Accept-Language headers. We're using a
 * Set for better performance in lookup.
 *
 * Example values: "en-US", "pt-BR"
 * @type {Set<string>}
 */
const SUPPORTED_LOCALE_SET = new Set(Object.values(SUPPORTED_LOCALES));

/**
 * Returns a valid locale from cookies, or null if not valid/found.
 * @param {{ get: (cookies: string) => any; }} cookies
 */
function localeFromCookies(cookies) {
  const locale = cookies.get('locale');
  return locale && SUPPORTED_LOCALE_SET.has(locale) ? locale : null;
}

/**
 * Parses the Accept-Language header and returns the best matching locale.
 * @param {string | null | undefined} header
 */
function localeFromHeader(header) {
  if (!header) return null;

  const parsedLanguages = parse(header);
  for (const { code, region } of parsedLanguages) {
    const locale = region ? `${code}-${region}` : code;
    if (SUPPORTED_LOCALE_SET.has(locale)) {
      return locale;
    }
  }

  return null;
}

/** @type {import('@sveltejs/kit').ServerLoad}*/
export async function load({ url, request, cookies }) {
  const cookieLocale = localeFromCookies(cookies);
  const headerLocale = localeFromHeader(request.headers.get('accept-language'));
  const language = cookieLocale || headerLocale || SUPPORTED_LOCALES.EN_US;
  const route = url.pathname;

  await loadTranslations(language, route);
  setLocale(language);
  setRoute(route);

  return { language, route };
}
